@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -21,7 +21,11 @@ import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import OrderedDict
 from hypothesis.internal.conjecture.utils import combine_labels
-from hypothesis.searchstrategy.strategies import MappedSearchStrategy, SearchStrategy
+from hypothesis.searchstrategy.strategies import (
+    MappedSearchStrategy,
+    SearchStrategy,
+    filter_not_satisfied,
+)
 
 
 class TupleStrategy(SearchStrategy):
@@ -87,13 +91,10 @@ class ListStrategy(SearchStrategy):
                 % (self.element_strategy,)
             )
         if self.element_strategy.is_empty and 0 < self.max_size < float("inf"):
-            from hypothesis._settings import note_deprecation
-
-            note_deprecation(
+            raise InvalidArgument(
                 "Cannot create a collection of max_size=%r, because no "
                 "elements can be drawn from the element strategy %r"
-                % (self.max_size, self.element_strategy),
-                since="2018-03-11",
+                % (self.max_size, self.element_strategy)
             )
 
     def calc_is_empty(self, recur):
@@ -146,13 +147,18 @@ class UniqueListStrategy(ListStrategy):
         seen = set()
         result = []
 
+        # We construct a filtered strategy here rather than using a check-and-reject
+        # approach because some strategies have special logic for generation under a
+        # filter, and FilteredStrategy can consolidate multiple filters.
+        filtered = self.element_strategy.filter(lambda val: self.key(val) not in seen)
         while elements.more():
-            value = data.draw(self.element_strategy)
-            k = self.key(value)
-            if k in seen:
+            value = filtered.filtered_strategy.do_filtered_draw(
+                data=data, filter_strategy=filtered
+            )
+            if value is filter_not_satisfied:
                 elements.reject()
             else:
-                seen.add(k)
+                seen.add(self.key(value))
                 result.append(value)
         assert self.max_size >= len(result) >= self.min_size
         return result
